<?php

declare(strict_types=1);


namespace FiloBlu\AdyenPaymentLinkGenerator\Block\Adminhtml\Order\Create\Search;

use Magento\Backend\Block\Context;
use Magento\Backend\Block\Widget\Grid\Column\Renderer\AbstractRenderer;
use Magento\Framework\DataObject;
use Magento\Framework\Module\Manager;
use Magento\Framework\ObjectManagerInterface;
use Magento\Store\Api\StoreRepositoryInterface;

/**
 *
 */
class Stock extends AbstractRenderer
{
    /** @var ObjectManagerInterface */
    protected $objectManager;

    /** @var StoreRepositoryInterface */
    private $storeRepository;

    /** @var Manager */
    protected $moduleManager;

    /**
     * @param Context $context
     * @param ObjectManagerInterface $objectManager
     * @param StoreRepositoryInterface $storeRepository
     * @param Manager $moduleManager
     * @param array $data
     */
    public function __construct(
        Context $context,
        ObjectManagerInterface $objectManager,
        StoreRepositoryInterface $storeRepository,
        Manager $moduleManager,
        array $data = []
    ) {
        $this->objectManager = $objectManager;
        $this->storeRepository = $storeRepository;
        $this->moduleManager = $moduleManager;
        parent::__construct($context, $data);
    }
    
    /**
     * Render review type
     *
     * @param \Magento\Framework\DataObject $row
     * @return \Magento\Framework\Phrase
     */
    public function render(DataObject $row)
    {
        if ($row->getData('store_id') == 0) {
            return '';
        }

        if ($this->moduleManager->isEnabled('FiloBlu_ExtInventory')) {
            /** @var \FiloBlu\ExtInventory\Helper\Data $extInventory */
            $extInventory = $this->objectManager->create(\FiloBlu\ExtInventory\Helper\Data::class);

            $store = $this->storeRepository->getById($row->getData('store_id'));
            $isSalable = $extInventory->isSalable($row->getData('sku'), $store->getWebsiteId());

            if ($isSalable) {
                return __('In Stock');
            }

            return __('Out Of Stock');
        }

        if ($row->getData('quantity_and_stock_status') > 0) {
            return __('In Stock');
        }

        return __('Out Of Stock');
    }
}
