<?php

namespace FiloBlu\AdyenPaymentLinkGenerator\Observer;

use Adyen\Payment\Helper\Data as AdyenHelper;
use FiloBlu\AdyenPaymentLinkGenerator\Helper\Data;
use Magento\Email\Model\BackendTemplate;
use Magento\Framework\App\Area;
use Magento\Framework\Event\Observer;
use Magento\Framework\Event\ObserverInterface;
use Magento\Framework\Mail\Template\TransportBuilder;
use Magento\Store\Model\Store;

class SendPBLNotification implements ObserverInterface
{
    const NOTIFY_PBL_EMAIL_TEMPLATE = 'notify_pbl_payment';

    /**
     * @var Data
     */
    protected $dataHelper;

    /**
     * @var AdyenHelper
     */
    protected $adyenHelper;

    /**
     * @var TransportBuilder
     */
    private $transportBuilder;

    public function __construct(
        Data             $dataHelper,
        AdyenHelper      $adyenHelper,
        TransportBuilder $transportBuilder
    )
    {
        $this->dataHelper = $dataHelper;
        $this->adyenHelper = $adyenHelper;
        $this->transportBuilder = $transportBuilder;
    }

    public function execute(Observer $observer)
    {
        $notificationRecipients = $this->dataHelper->getNotificationRecipientsArray();
        if (!$this->dataHelper->sendPBLNotificationIsEnabled()
            || $observer->getEvent()->getNotificationCode() != 'AUTHORISATION'
            || empty($notificationRecipients)) {
            return;
        }

        $order = $observer->getEvent()->getOrder();
        $payment = $order->getPayment();
        if (is_null($payment->getAdditionalInformation('pay_by_link_data'))) {
            return;
        }

        $payByLinkData = json_decode($payment->getAdditionalInformation('pay_by_link_data'), true);

        $client = $this->adyenHelper->initializeAdyenClient();
        $service = $this->adyenHelper->createAdyenCheckoutService($client);

        foreach ($payByLinkData as $key => $link) {
            if ($link['notify']) {
                continue;
            }

            $pblStatus = $service->retrievePaymentLinks($link['link_id']);
            if (empty($pblStatus)
                || $pblStatus['status'] != 'completed'
                || $pblStatus['reference'] != $order->getIncrementId()) {
                continue;
            }

            $transport = $this->transportBuilder->setTemplateIdentifier(
                self::NOTIFY_PBL_EMAIL_TEMPLATE
            )
                ->setTemplateModel(BackendTemplate::class)
                ->setTemplateOptions([
                        'area' => Area::AREA_ADMINHTML,
                        'store' => Store::DEFAULT_STORE_ID
                    ]
                )
                ->setTemplateVars([
                    'orderid' => $pblStatus['reference'],
                    'linkid' => $pblStatus['id']
                ])
                ->setFromByScope([
                    'email' => $this->dataHelper->emailSender(),
                    'name' => $this->dataHelper->nameSender()
                ])
                ->addTo($notificationRecipients)
                ->getTransport();

            $transport->sendMessage();

            $link['notify'] = 1;
            $payByLinkData[$key] = $link;
            $payment->setAdditionalInformation('pay_by_link_data', json_encode($payByLinkData));
        }
    }
}
